import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal, max, format, extent } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';
import ReactDropdown from 'react-dropdown';
import crossfilter from 'crossfilter';

const width = 960;
const height = 500;
const margin = { top: 20, right: 200, bottom: 150, left: 100 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;
const fadeOpacity = 0.2;

const xyAttributes = [
 
  { value: 'salary', label: 'Salary' },
  { value: 'yrs.since.phd', label: 'Years since PhD' },
  { value: 'yrs.service', label: 'Years of service' }

 ];
const filterAttributes = [
  { value: 'rank', label: 'Rank' },
  { value: 'discipline', label: 'Discipline' },
    { value: 'sex', label: 'Sex' }  ];

const getAxisLabel = value => {
  for (let i = 0; i < xyAttributes.length; i++) {
    if (xyAttributes[i].value === value) {
      return xyAttributes[i].label;
    }
  }
};
const getFilterLabel = value => {
  for (let i = 0; i < filterAttributes.length; i++) {
    if (filterAttributes[i].value === value) {
      return filterAttributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();
  const initialXAttribute = 'yrs.service';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getAxisLabel(xAttribute);

  const initialYAttribute = 'salary';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getAxisLabel(yAttribute);
  
  const initialFilterAttribute = 'rank';
  const [filterAttribute, setFilterAttribute] = useState(initialFilterAttribute);
  const colorValue = d => d[filterAttribute];
  const colorLegendLabel = getFilterLabel(filterAttribute);
  
  
  const [hoveredValue, setHoveredValue] = useState(null);

  if (!data) {
    return <pre>Loading...</pre>;
  }
  var fildata= crossfilter(data);
  var n=fildata.groupAll().reduceCount().value();
  var df=fildata.dimension(d=>d.discipline);
  var sal = df.filter("A").top(Infinity);
  var ghj=df.group().reduceCount().all();
  df.filter("B");
console.log(data);
  console.log(ghj);
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

 
  const filteredData = data.filter(d => hoveredValue === colorValue(d));

  const circleRadius = 7;

  const siFormat = format('0.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue);
  const yAxisTickFormat = tickValue => siFormat(tickValue);

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([ innerHeight,0]);

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#04f000','#fab300','#f0f000']);

  return (
    <>
    <div className="menus-container">
        <span className="dropdown-label">X</span>
        <ReactDropdown
          options={xyAttributes}
          value={xAttribute}
          onChange={({ value }) => setXAttribute(value)}
        />
        <span className="dropdown-label">Y</span>
        <ReactDropdown
          options={xyAttributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      <span className="dropdown-label">Filter</span>
        <ReactDropdown
          options={filterAttributes}
          value={filterAttribute}
          onChange={({ value }) => setFilterAttribute(value)}
        />
      </div>
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={5}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${innerHeight /
            2}) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
        <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5} tickFormat={yAxisTickFormat} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <g transform={`translate(${innerWidth + 60}, 60)`}>
          <text x={35} y={-25} className="axis-label" textAnchor="middle">
            {colorLegendLabel}
          </text>
          <ColorLegend
            tickSpacing={22}
            tickSize={10}
            tickTextOffset={12}
            tickSize={circleRadius}
            colorScale={colorScale}
            onHover={setHoveredValue}
            hoveredValue={hoveredValue}
            fadeOpacity={fadeOpacity}
          />
        </g>
        <g opacity={hoveredValue ? fadeOpacity : 1}>
          <Marks
            data={data}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
            colorValue={colorValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
          />
        </g>
        <Marks
          data={filteredData}
          xScale={xScale}
          xValue={xValue}
          yScale={yScale}
          yValue={yValue}
          colorScale={colorScale}
          colorValue={colorValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={circleRadius}
        />
      </g>
    </svg>
      </>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
